﻿using ICSharpCode.SharpZipLib.Core;
using ICSharpCode.SharpZipLib.Zip;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace mxvalheim
{
    public class Root
    {
        public string name { get; set; }
        public string url { get; set; }
    }
    class Program
    {
        static void Main(string[] args)
        {
            WebClient wc = new WebClient();
            
            if (!Directory.Exists(@"tmp")) { Directory.CreateDirectory("tmp"); }

            if (File.Exists(@"tmp\plugins.zip")) { File.Delete(@"tmp\plugins.zip"); }
            if (File.Exists(@"tmp\config.zip")) { File.Delete(@"tmp\config.zip"); }

            wc.DownloadFile("https://io.mxg.ovh/valheim/mxval/plugins.zip", $"tmp/plugins.zip");
            wc.DownloadFile("https://io.mxg.ovh/valheim/mxval/config.zip", $"tmp/config.zip");

            ExtractZipContent($"tmp/plugins.zip", null, $@"BepInEx");
            ExtractZipContent($"tmp/config.zip", null, $@"BepInEx");
        }

        public static void ExtractZipContent(string FileZipPath, string password, string OutputFolder)
        {
            ICSharpCode.SharpZipLib.Zip.ZipFile file = null;
            try
            {
                FileStream fs = File.OpenRead(FileZipPath);
                file = new ICSharpCode.SharpZipLib.Zip.ZipFile(fs);

                if (!String.IsNullOrEmpty(password))
                {
                    // AES encrypted entries are handled automatically
                    file.Password = password;
                }

                foreach (ZipEntry zipEntry in file)
                {
                    if (!zipEntry.IsFile)
                    {
                        // Ignore directories
                        continue;
                    }

                    String entryFileName = zipEntry.Name;
                    // to remove the folder from the entry:- entryFileName = Path.GetFileName(entryFileName);
                    // Optionally match entrynames against a selection list here to skip as desired.
                    // The unpacked length is available in the zipEntry.Size property.

                    // 4K is optimum
                    byte[] buffer = new byte[4096];
                    Stream zipStream = file.GetInputStream(zipEntry);

                    // Manipulate the output filename here as desired.
                    String fullZipToPath = Path.Combine(OutputFolder, entryFileName);
                    string directoryName = Path.GetDirectoryName(fullZipToPath);

                    if (directoryName.Length > 0)
                    {
                        Directory.CreateDirectory(directoryName);
                    }

                    // Unzip file in buffered chunks. This is just as fast as unpacking to a buffer the full size
                    // of the file, but does not waste memory.
                    // The "using" will close the stream even if an exception occurs.
                    using (FileStream streamWriter = File.Create(fullZipToPath))
                    {
                        StreamUtils.Copy(zipStream, streamWriter, buffer);
                    }
                }
            }
            finally
            {
                if (file != null)
                {
                    file.IsStreamOwner = true; // Makes close also shut the underlying stream
                    file.Close(); // Ensure we release resources
                }
            }
        }
    }   
}
